SystemOrganization addCategory: #'Package-Base'!
SystemOrganization addCategory: #'Package-Test'!

!MethodReference methodsFor: '*Package-package browser' stamp: 'ab 8/28/2005 10:17'!
classSymbol: aSymbol
	classSymbol := aSymbol! !

!MethodReference methodsFor: '*Package-package browser' stamp: 'ab 2/22/2005 17:55'!
packageDefining
	| ps |
	ps := Package allPackages select: [:p| p includesMethod: self methodSymbol 
											ofClass: self actualClass].
	^ ps ifEmpty: [nil] ifNotEmpty: [ ps first]! !

!MCPackage methodsFor: '*Package-as yet unclassified' stamp: 'alexandrebergel 11/2/2005 12:05'!
snapshot
	| package definitions categories methodDefinitions |
	package := self packageInfo.
	definitions := OrderedCollection new.
	categories := package systemCategories.
	categories isEmpty ifFalse: [ definitions add: (MCOrganizationDefinition categories: categories) ].

	methodDefinitions := OrderedCollection new.
	package methods do: [:ea | methodDefinitions add: ea asMethodDefinition] displayingProgress: 'Snapshotting methods...'.

	"Insure compatibility while using PackageBrowser"
	methodDefinitions do: [:ea| 
		(ea isClassExtensionOf: package) 
			ifTrue: ["We check if the category does not already follow the old contention"
					(ea category first ~= $*)  
						ifTrue: ["If not, then we add *Package in front of"
								ea category: ('*', package name, '-', ea category)]]]. 

	definitions addAll: methodDefinitions.

	(package respondsTo: #overriddenMethods) ifTrue:
		[package overriddenMethods
			do: [:ea | definitions add:
					(package changeRecordForOverriddenMethod: ea) asMethodDefinition]
			displayingProgress: 'Searching for overrides...'].
	package classes do: [:ea | definitions addAll: ea classDefinitions] displayingProgress: 'Snapshotting classes...'.
	
	package hasComment ifTrue: 
		[definitions add: (MCPackageCommentDefinition fromPackage: package)].
	^ MCSnapshot fromDefinitions: definitions
! !

!AddedEvent class methodsFor: '*Package-accessing' stamp: 'ab 2/14/2005 19:38'!
supportedKinds
	"All the kinds of items that this event can take."
	
	^ Array with: self classKind with: self methodKind with: self categoryKind with: self protocolKind with: self packageKind! !

!MCMczWriter methodsFor: '*Package-serializing' stamp: 'ab 7/7/2005 15:10'!
serializePackage: aPackage
	| comment |

	comment := aPackage packageInfo hasComment
					ifTrue: [' comment ''', aPackage packageInfo comment, '''']
					ifFalse: [''].

	^ '(name ''', aPackage name, '''', comment, ')'! !

Object subclass: #Package
	instanceVariableNames: 'name classes methods description cache'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Package-Base'!

Package class
	instanceVariableNames: 'packages'!

!Package commentStamp: 'ab 2/8/2005 14:32' prior: 0!
I represent a unit of modularization. 

Structure:
 name			String -- Package name
 classes			Collection of classes -- Class references that belong to the package
 methods		Collection of MethodReference -- Instances of MethodReference refering to methods that belong to the package. 

Method category syncWithSystem contains a lot of method from PackageInfo. It is useful to import package from the system that uses packageinfo convention!

!Package class methodsFor: 'packages' stamp: 'ab 8/28/2005 14:30'!
allPackages
	^ self packages values! !

!Package class methodsFor: 'installing' stamp: 'ab 8/28/2005 14:30'!
bootstrap
	"Replace packageInfo with packages"

	self hookIntoTheSystem.
	self migratePackageInfo.
	self makePackagesFromSqueak.
	self getNotified.! !

!Package class methodsFor: 'classes and packages' stamp: 'ab 8/28/2005 14:30'!
classesAndPackages
	^ classesAndPackages ifNil: [classesAndPackages := IdentityDictionary new]! !

!Package class methodsFor: 'updating' stamp: 'ab 8/28/2005 15:35'!
event: anEvent
	"Hook for SystemChangeNotifier"

	| cls pak pakName |

"Transcript show: anEvent; cr.
"
	(anEvent isAdded and: [anEvent itemKind = SystemChangeNotifier classKind]) 
		ifTrue: [| c p |
				c := anEvent item.
				pakName := (c category findTokens: $-) first.
				"Note that the package is created if it does not exist"
				p := Package named: pakName.
				p addClass: c. ^self].

	cls := anEvent itemClass.
	cls ifNil: [^ self].

	pak := cls package.
	pak ifNil: [^ self].

	pak invalidateCache.

	(anEvent isRemoved 
		and: [anEvent itemKind = SystemChangeNotifier classKind])
			ifTrue: [pak removeClassNamed: cls name].

	((anEvent isRemoved 
		and: [anEvent itemKind = SystemChangeNotifier methodKind])
			and: [pak includesMethod: anEvent itemSelector ofClass: cls])
				ifTrue: [pak removeMethodNamed: anEvent itemSelector forClass: cls].

	(anEvent isAdded 
		and: [anEvent itemKind = SystemChangeNotifier methodKind])
				ifTrue: [ |packag sel|
					sel := anEvent itemSelector.
					packag := self packageBelongingMethodNamed: sel inClass: cls.
					packag addMethod: (cls>> sel) methodReference].

	((anEvent isRecategorized or: [anEvent isModified])
		and: [anEvent itemKind = SystemChangeNotifier methodKind])
				ifTrue: [ | oldPackag newPackag cm mr sel|
					cm := anEvent item.
					mr := cm methodReference.
					sel := mr methodSymbol.
"(sel = #zork) ifTrue: [self halt].
"					oldPackag := cm methodReference packageDefining.
					newPackag := self packageBelongingMethodNamed: sel inClass: cls.
					(oldPackag == newPackag) ifTrue: [^ self].
					oldPackag 
						ifNil: [newPackag addMethod: cm]
						ifNotNil: [oldPackag moveMethods: cm methodReference to: newPackag]].


	(anEvent isRenamed 
		and: [anEvent itemKind = SystemChangeNotifier classKind])
				ifTrue: [self renameClass: anEvent oldName to: anEvent newName].
! !

!Package class methodsFor: 'testing' stamp: 'ab 8/28/2005 14:31'!
exist: aName
	^ self packages includesKey: aName! !

!Package class methodsFor: 'instance creation' stamp: 'ab 8/28/2005 14:30'!
fromCategory: categoryName
	| classes package |
	classes := SystemOrganization listAtCategoryNamed: categoryName.
	package := self named: categoryName.
	package classes: classes.
	package compiledMethods: 
		(classes inject: OrderedCollection new into: 
			[:sum :el| sum, el methodDictionary values]).
	^ package! !

!Package class methodsFor: 'instance creation' stamp: 'ab 8/28/2005 14:30'!
fromPackageInfo: packageInfo

	^ (self named: packageInfo packageName) initFromPackageInfo: packageInfo! !

!Package class methodsFor: 'updating' stamp: 'ab 8/28/2005 14:30'!
getNotified
	SystemChangeNotifier uniqueInstance notify: self ofAllSystemChangesUsing: #event:.! !

!Package class methodsFor: 'installing' stamp: 'ab 8/28/2005 14:31'!
hookIntoTheSystem
	MCPackage compile: 'packageInfo
	^ Package named: name'.

"	BogusInfo class compile: 'package
	^ Package hidden: self packageName'.	"

	"------------"

	MCWorkingCopyBrowser compile: 'addWorkingCopy
	|name|
	name _ FillInTheBlankMorph request: ''Name of package:''.
	name isEmptyOrNil ifFalse:
		[Package registerPackageName: name.
		workingCopy _ MCWorkingCopy forPackage: (MCPackage new name: name).
		workingCopyWrapper _ nil.
		self repositorySelection: 0].
	self workingCopyListChanged; changed: #workingCopySelection; changed: #repositoryList'.

	"------------"

	(Smalltalk includesKey: #MCWorkingCopyList) ifTrue: [
		(Smalltalk at: #MCWorkingCopyList) compile: 'addWorkingCopy
	|name|
	name _ FillInTheBlankMorph request: ''Name of package:''.
	name isEmptyOrNil ifFalse:
		[Package registerPackageName: name.
		workingCopy _ MCWorkingCopy forPackage: (MCPackage new name: name).
		self repositorySelection: 0].
	self changed: #workingCopyList; changed: #workingCopySelection; changed: #repositoryList'].

	"------------"

	PackageOrganizer compile: 'registerPackageNamed: aString
	^ self registerPackage: (Package named: aString)'! !

!Package class methodsFor: 'installing' stamp: 'ab 8/28/2005 14:30'!
initialize

self allInstances 
	ifEmpty: [self bootstrap]
	ifNotEmptyDo: [:instances |
		instances do: [:ea | ea updateInstVars].
		"self rescanForStarredCategories"].! !

!Package class methodsFor: 'installing' stamp: 'ab 8/28/2005 14:30'!
makePackagesFromSqueak

	| packageNames |

	(SystemVersion current version beginsWith: 'Squeak3.9') ifTrue: [
		"squeak 3.9 is already packagized -- decline"
		^ self].

	packageNames _ OrderedCollection new.
	packageNames addAll: 
			(SystemOrganization categories collect: [:c | 
				| stem |
				stem _ ((c indexOf: $-) > 0) ifFalse: [c] ifTrue: [c copyFrom: 1 to: ((c indexOf: $-) -1)].
				stem asSymbol]).

	'Packagizing Squeak categories' 
		displayProgressAt: Display center from: 1 to: packageNames size during: [:bar | 
			1 to: packageNames size do: [:i | | pname |
				bar value: i. 
				pname _ packageNames at: i.
				self packages at: pname ifAbsent: [
					(PackageInfo new packageName: pname) asPackage.
					"create a working copy, to register the package with the monticello browser"
					(MCPackage named: pname) workingCopy]]]! !

!Package class methodsFor: 'installing' stamp: 'ab 8/28/2005 14:30'!
migratePackageInfo
	| porg pinfos paks |
	porg := PackageOrganizer default.
	pinfos := porg packages select: [:p | p isKindOf: PackageInfo].

	paks := OrderedCollection new.
	'Migrating PackageInfo instances' displayProgressAt: Display center
		from: 1 to: pinfos size
		during: [:bar | 1 to: pinfos size do: [:i | 
			bar value: i. 
			paks add:  (pinfos at: i) asPackage]].

	paks do: [:a| self packages at: a packageName put: a].
	pinfos do: [:pp| porg unregisterPackage: pp].
	self packages values do: [:pp| porg registerPackage: pp].! !

!Package class methodsFor: 'instance creation' stamp: 'ab 8/28/2005 14:30'!
named: aName
	| p |
	self packages at: aName ifPresent: [:pak| ^ pak].

	p := self new.
	p name: aName.
	self packages at: aName put: p.
	SystemChangeNotifier uniqueInstance packageAdded: p.
	^ p
	! !

!Package class methodsFor: 'as yet unclassified' stamp: 'ab 8/28/2005 14:30'!
nil! !

!Package class methodsFor: 'updating' stamp: 'ab 8/28/2005 14:30'!
packageBelongingMethodNamed: selector inClass: cls

	| pakName cat |
	cat := cls organization categoryOfElement: selector.
	(cat first = $*)
		ifTrue: [|i|
				i := cat indexOf: $-.
				(i = 0) 
					ifTrue: [pakName := cat allButFirst]
					ifFalse: [pakName := cat copyFrom: 2 to: (i - 1)].
				^ Package named: pakName].
	^ cls package! !

!Package class methodsFor: 'classes and packages' stamp: 'ab 8/28/2005 14:30'!
packageDefining: aClass
	packages associationsDo: [:assoc|
		(assoc value includesClass: aClass)
			ifTrue: [^ assoc value]].
	^ nil! !

!Package class methodsFor: 'packages' stamp: 'ab 8/28/2005 14:30'!
packages
	^ packages ifNil: [packages := Dictionary new]! !

!Package class methodsFor: 'packages' stamp: 'ab 8/28/2005 14:30'!
registerPackageName: aString
	^ self named: aString! !

!Package class methodsFor: 'removing' stamp: 'ab 8/28/2005 14:30'!
removeIfExist: aName

	self packages removeKey: aName ifAbsent: []
! !

!Package class methodsFor: 'removing' stamp: 'ab 8/28/2005 14:30'!
remove: aName

	| p |
	p := self packages at: aName.
	self packages removeKey: aName ifAbsent: [self error: 'Package not found ', aName].

	SystemChangeNotifier uniqueInstance packageRemoved: p.
! !

!Package class methodsFor: 'removing' stamp: 'ab 8/28/2005 14:30'!
remove: aName ifAbsent: aBlock

	self packages removeKey: aName ifAbsent: aBlock! !

!Package class methodsFor: 'updating' stamp: 'ab 8/28/2005 14:30'!
renameClass: oldName to: newName
	| ps pp|

	ps := self packages select: [:p| p doesDefineMethodForClassNamed: oldName]. 
	ps associationsDo: [:assoc | pp := assoc value.
								(pp methodsLocallyDefinedForClassNamed: oldName) 
									do: [:m| m classSymbol: newName]].! !

!Package class methodsFor: 'installing' stamp: 'ab 8/28/2005 14:30'!
rescanForStarredCategories
"if the user has created method extensions with packageInfo conventions inside an already bootstrapped image, fold those extensions back into the intended package"

	self notYetImplemented! !

!Package methodsFor: 'adding' stamp: 'ab 8/28/2005 09:43'!
addClass: cls
	self assert: [ cls isBehavior ].
	(self classes includes: cls) not 
		ifTrue: [
			self classes add: cls.
			self adjustCategoryFor: cls]! !

!Package methodsFor: 'adding' stamp: 'ab 7/11/2005 12:41'!
addMethod: method
	"method should be instance of a compiled method or of MethodReference"	

	method isCompiledMethod 
		ifTrue: [^ self addMethod: method methodReference.].

	self assert: [method isKindOf: MethodReference].

	"We ignore DoIt, normaly the doit mechanism should do it for us, but not always apparently"
	(method methodSymbol == #DoIt or: [method methodSymbol == #DoItIn:]) ifTrue: [^ self].

	self invalidateCache.
	self methods add: method.
! !

!Package methodsFor: 'adding' stamp: 'ab 3/24/2005 18:20'!
adjustCategoryFor: cls
	| catArray newCat |
	catArray := cls category subStrings: {$-}.
	catArray at: 1 put: self name.
	newCat := catArray inject: '' into: [:sum :el| sum, el, '-'].
	cls category: newCat allButLast! !

!Package methodsFor: 'requirement' stamp: 'ab 7/8/2005 08:15'!
anyPackageRequired
	"Return true or false"

	^ self mcWorkingCopy anyPackageRequired! !

!Package methodsFor: 'system categories' stamp: 'mist 8/22/2005 17:28'!
category: categoryName matches: prefix
	^ categoryName notNil and: [categoryName = prefix or: [categoryName beginsWith: prefix, '-']]! !

!Package methodsFor: 'as yet unclassified' stamp: 'ab 4/10/2005 19:15'!
classes

	^ classes ifNil: [classes := OrderedCollection new]! !

!Package methodsFor: 'sub category' stamp: 'ab 5/16/2005 22:57'!
classesOfSubcategory: subCategoryName
	"subCategoryName can be '' or a name"
	
	| cat classNames |
	cat := (subCategoryName = '') 
		ifTrue: [ self name ]
		ifFalse: [ self name, '-', subCategoryName ].

	classNames := SystemOrganization listAtCategoryNamed: cat.
	^ classNames collect: [:cname| self environment at: cname].

"	^ self classes select: [:cls | cls category = cat]
"! !

!Package methodsFor: 'as yet unclassified' stamp: 'ab 2/5/2005 17:02'!
classes: collectionOfClasses
	classes := collectionOfClasses! !

!Package methodsFor: 'as yet unclassified' stamp: 'ab 3/24/2005 15:41'!
classNamed: aSymbol
	| a |
	a := self classes select: [:cls | cls name == aSymbol].
	a ifEmpty: [^self error: 'Class ', aSymbol, ' does not exist'].

	^ a first.
	! !

!Package methodsFor: 'as yet unclassified' stamp: 'ab 8/28/2005 09:54'!
cleanUpOrphanMethod
	self methods removeAllSuchThat: [:mr| mr actualClass isNil ]! !

!Package methodsFor: 'comment' stamp: 'mist 8/23/2005 23:40'!
comment
	^ description! !

!Package methodsFor: 'comment' stamp: 'mist 8/23/2005 23:40'!
comment: aComment
	description := aComment! !

!Package methodsFor: 'as yet unclassified' stamp: 'ab 2/5/2005 17:02'!
compiledMethods: collectionOfCompiledMethods
	compiledMethods := collectionOfCompiledMethods! !

!Package methodsFor: 'class extension' stamp: 'ab 8/28/2005 10:10'!
doesDefineMethodForClassNamed: clsName
	^ self methods anySatisfy: [:mr| mr classSymbol == clsName]! !

!Package methodsFor: 'class extension' stamp: 'ab 5/12/2005 15:52'!
doesDefineMethodForClass: cls
	^ self methods anySatisfy: [:mr| mr actualClass == cls]! !

!Package methodsFor: 'as yet unclassified' stamp: 'ab 4/26/2005 21:37'!
environment
	^ Smalltalk! !

!Package methodsFor: 'comment' stamp: 'mist 8/23/2005 23:41'!
hasComment
	^ description notNil! !

!Package methodsFor: 'testing' stamp: 'ab 2/6/2005 15:43'!
includesClassNamed: aClassName
	^ (self classes select: [:cls| cls name == aClassName]) notEmpty! !

!Package methodsFor: 'testing' stamp: 'ab 5/12/2005 14:10'!
includesClass: aClass
	self assert: (aClass isBehavior).
	^ self classes includes: aClass theNonMetaClass! !

!Package methodsFor: 'mcBrowser compatibility ' stamp: 'ab 4/10/2005 19:13'!
includesMethodCategory: aSymbol ofClass: aClass 
	^ false! !

!Package methodsFor: 'testing' stamp: 'ab 2/6/2005 16:35'!
includesMethod: aSymbol ofClass: aClass
	aClass ifNil: [^ false].
	^ self methods anySatisfy: 
			[:mf| (mf actualClass == aClass) and: [mf methodSymbol == aSymbol]]! !

!Package methodsFor: 'system categories' stamp: 'mist 8/22/2005 17:29'!
includesSystemCategory: categoryName
	^ self category: categoryName matches: self name! !

!Package methodsFor: 'michal' stamp: 'mist 8/24/2005 17:39'!
initFromPackageInfo: pi

	| newClasses |
	newClasses _ pi systemCategories gather: [:cat |
					(SystemOrganization listAtCategoryNamed: cat) collect: [:className | Smalltalk at: className]].
	classes _ newClasses asOrderedCollection.
	methods _ (pi coreMethods, pi extensionMethods) asOrderedCollection.! !

!Package methodsFor: 'as yet unclassified' stamp: 'mist 8/23/2005 23:45'!
initialize

	super initialize.

	cache _ PackageExtensionsCache new.! !

!Package methodsFor: 'monticello' stamp: 'ab 5/23/2005 16:03'!
installMethodDefinition: def
	| cls mr cat |
	cls := def actualClass.
	cls ifNil: [Transcript cr; show: 'Class ', def className, 
				' is not present in the system, method ', def selector, ' skipped'. 
				^ self].

	mr := MethodReference class: cls selector: def selector.

	cat := mr category.
	(cat first = $*)
		ifTrue: [|newCat|
				newCat := (cat indexOf: $-) = 0
								ifTrue: [cat allButFirst]
								ifFalse: [cat copyAfter: $-].
					cls organization 
						classify: mr methodSymbol under: newCat suppressIfDefault: true].

	"We check for a class extension"
	(self includesMethod: def selector ofClass: def actualClass) 
		ifFalse: [self addMethod: mr]! !

!Package methodsFor: 'cache' stamp: 'mist 8/23/2005 23:51'!
invalidateCache
	cache invalidate! !

!Package methodsFor: 'requirement' stamp: 'ab 7/8/2005 08:16'!
isPackageRequired: aPackage
	"Return true or false"

	^ self mcWorkingCopy isPackageRequired: aPackage! !

!Package methodsFor: 'monticello' stamp: 'ab 2/6/2005 19:45'!
lastCommit
	^ self mcWorkingCopy ancestry ancestorString! !

!Package methodsFor: 'class extension' stamp: 'mist 8/24/2005 00:01'!
listOfExtendedClasses
	"Return an array of classes being extended by the receiver"

	^ cache classExtensions ifNil: [cache recordClassExtensionsFor: self]! !

!Package methodsFor: 'class extension' stamp: 'ab 3/21/2005 11:20'!
listOfMethodExtensions

	^ (self methodExtensions collect: [:mr | |cls| cls := mr actualClass.
									{cls package . cls . mr methodSymbol}]) asArray! !

!Package methodsFor: 'class extension' stamp: 'ab 5/12/2005 15:41'!
listOfMethodExtensionsFor: aClass

	| a tt |
	self methodExtensionsCache at: aClass name ifPresent: [:v| ^ v].

	a := self listOfMethodExtensions.
	tt := (a select: [:t| t second == aClass]) collect: [:t| t third].
	self methodExtensionsCache at: aClass name put: tt.
	^ tt
! !

!Package methodsFor: 'monticello' stamp: 'ab 7/8/2005 08:05'!
mcWorkingCopy

	^ (MCPackage named: self packageName) workingCopy! !

!Package methodsFor: 'as yet unclassified' stamp: 'ab 5/12/2005 11:53'!
methodExtensions
	| clss |
	clss := self classes.
	^ (self methods 
		reject: [:mr| mr actualClass isNil])
		reject: [:mr | (clss includes: mr actualClass theNonMetaClass) or: [mr actualClass isNil]]! !

!Package methodsFor: 'cache' stamp: 'mist 8/23/2005 23:47'!
methodExtensionsCache
	"Class Names as keys, and list of methods as values"

	^ cache methodExtensions! !

!Package methodsFor: 'as yet unclassified' stamp: 'ab 2/6/2005 16:36'!
methods
	^ methods ifNil: [methods := OrderedCollection new]! !

!Package methodsFor: 'class extension' stamp: 'ab 8/28/2005 10:11'!
methodsLocallyDefinedForClassNamed: clsName
	^ self methods select: [:mr| mr classSymbol == clsName]! !

!Package methodsFor: 'class extension' stamp: 'ab 5/12/2005 12:40'!
methodsLocallyDefinedForClass: cls
	^ self methods select: [:mr| mr actualClass == cls]! !

!Package methodsFor: 'michal' stamp: 'mist 8/24/2005 14:27'!
methodsWithClassSymbol: existingSymbol
"for debugging"

^ methods select: [:m | m classSymbol == existingSymbol]! !

!Package methodsFor: 'michal' stamp: 'mist 8/24/2005 14:31'!
methodsWithClassSymbol: existingSymbol become: newClass
"for debugging"

(self methodsWithClassSymbol: existingSymbol) do: [:m | m setStandardClass: newClass methodSymbol: m methodSymbol]! !

!Package methodsFor: 'as yet unclassified' stamp: 'ab 2/5/2005 17:02'!
methods: collectionOfCompiledMethods
	methods := collectionOfCompiledMethods! !

!Package methodsFor: 'monticello' stamp: 'ab 2/6/2005 19:44'!
modified
	^ self mcWorkingCopy modified! !

!Package methodsFor: 'sub category' stamp: 'ab 3/24/2005 19:16'!
moveClassNamed: className inSubcategoryNamed: categoryName
	| cls oldCatArray newCat |

	cls := self classNamed: className.

	(categoryName = '') 
		ifTrue: [cls category: self name]
		ifFalse: [
			oldCatArray := cls category subStrings: {$-}.
			(oldCatArray size = 1) ifTrue: [cls category: (self name, '-', categoryName). ^self].

			oldCatArray := oldCatArray allButLast.
			newCat := (oldCatArray, {categoryName}) inject: '' into: [:sum :el| sum, el, '-'].
			newCat := newCat allButLast.
			cls category: newCat]! !

!Package methodsFor: 'operators' stamp: 'ab 7/11/2005 12:42'!
moveMethods: aMethodReference to: aPackage
"	(self includesMethod: aMethodReference methodSymbol 
		 ofClass: aMethodReference actualClass )
		ifFalse: [self error: 'Method not visible'].
"

	| packageDefiningMethod |
	"There is no reference to self because a package that does not define a method may require to move it, have a look at the last part of the test testMoveMethod"

	"often packageDefiningMethod == self, but it may not"
	packageDefiningMethod := aMethodReference packageDefining.

	packageDefiningMethod ifNotNil: [
		packageDefiningMethod methods 
			removeAll: (packageDefiningMethod methods select: [:mr| mr = aMethodReference]).
			packageDefiningMethod invalidateCache].
	aPackage addMethod: aMethodReference.

	SystemChangeNotifier uniqueInstance 
		methodRemoved: aMethodReference compiledMethod selector: aMethodReference methodSymbol class: aMethodReference actualClass 
! !

!Package methodsFor: 'as yet unclassified' stamp: 'ab 2/6/2005 15:54'!
name
	^ name! !

!Package methodsFor: 'as yet unclassified' stamp: 'ab 7/7/2005 15:21'!
name: aName
	name := aName asString! !

!Package methodsFor: 'as yet unclassified' stamp: 'ab 2/5/2005 17:10'!
packageName
	^ name! !

!Package methodsFor: 'class extension' stamp: 'ab 5/12/2005 15:52'!
packagesExtendingClass: cls
	^ ((Package packages values copyWithout: self)
		select: [:p| p doesDefineMethodForClass: cls]) asArray! !

!Package methodsFor: 'printing' stamp: 'ab 3/23/2005 14:43'!
printOn: stream

	super printOn: stream.
	stream nextPutAll: '(', self name, ')'! !

!Package methodsFor: 'removing' stamp: 'ab 2/9/2005 23:18'!
removeClassNamed: aName
	| cls |
	cls := (self classes select: [:c| c name == aName]) first.
	self classes remove: cls.
	self methods removeAll: (self methods select: [:mr| mr classSymbol == aName])! !

!Package methodsFor: 'removing' stamp: 'ab 8/28/2005 15:22'!
removeFromSystem
	| classesRemoved t |

	self classes do: [:cls|
		(self packagesExtendingClass: cls) do: [:p| p removeMethodsForClass: cls]].

	self cleanUpOrphanMethod.

	self mcWorkingCopy unload.

	self methods do: [:mr| |c|
		c := mr actualClass.
		c ifNotNil: [c removeSelector: mr methodSymbol]].
	classesRemoved := OrderedCollection new.
	self classes do: [:cls|
		classesRemoved add: cls name.
		cls removeFromSystem].

	"-----"
	"In case if some classes were not properly inserted..."
	t := (SystemOrganization listAtCategoryNamed: self name) copy.
	t ifNotEmpty: [
		t removeAll: classesRemoved.
		t do: [:clsname| Smalltalk at: clsname ifPresent: [:c| c removeFromSystem]]].
	"-----"

	"We remove subcategories"
	self systemCategories do: [:cat| SystemOrganization removeCategory: cat].

"(self name = 'MockPackage') ifTrue: [self halt].
"
	"We remove the category corresponding to the name of this package."
	(SystemOrganization categories includes: self name)
		ifTrue: [SystemOrganization removeCategory: self name].

	self unregister.
	SystemChangeNotifier uniqueInstance packageRemoved: self! !

!Package methodsFor: 'removing' stamp: 'ab 7/11/2005 12:41'!
removeMethodNamed: aName forClass: aClass
	| mr mrToRemove |
	mr := MethodReference class: aClass selector: aName.
	mrToRemove := (self methods select: [:m| m = mr]) first.
	self invalidateCache.
	self methods remove: mrToRemove.
! !

!Package methodsFor: 'class extension' stamp: 'ab 5/12/2005 15:56'!
removeMethodsForClass: cls
	"Called when unstalling a package that contains extended class. When installing it, method extensions of this class (which belongs to other packages) have to be removed either. This is what this method is for"

	self methods removeAllSuchThat: [:mr| mr actualClass == cls]! !

!Package methodsFor: 'operators' stamp: 'ab 8/28/2005 15:32'!
rename: aName
	| cat t oldName subCategories |
	oldName := self name.
	subCategories := self subcategories.

	self class remove: self name.
	self name: aName.
	self class packages at: aName put: self.

	self classes do: [:cls|
		cat := cls category.
		t := cat findTokens: $-.
		t removeFirst.
		t addFirst: aName.
		cls category: (t inject: '' into: [:sum :el| sum, '-', el]) allButFirst].

	"We remove categories"
	(subCategories, (Array with: oldName)) do: [:ca| 
		(SystemOrganization categories includes: ca)
			ifTrue: [SystemOrganization removeCategory: ca]].

	SystemChangeNotifier uniqueInstance packageRenamed: self.! !

!Package methodsFor: 'monticello' stamp: 'ab 2/6/2005 19:40'!
repositories

	^ self mcWorkingCopy repositoryGroup repositories
! !

!Package methodsFor: 'requirement' stamp: 'ab 7/8/2005 08:06'!
requirePackage: aPackage
	self mcWorkingCopy requirePackage: aPackage! !

!Package methodsFor: 'sub category' stamp: 'ab 3/24/2005 11:54'!
subCategoryOf: className
	| cls ans |
	cls := self classNamed: className.	
	ans := (cls category subStrings: {$-}) last.
	^ (ans = self name)
		ifTrue: ['']
		ifFalse: [ans]! !

!Package methodsFor: 'sub category' stamp: 'ab 4/10/2005 19:46'!
subcategories
	| ans |
	ans := (SystemOrganization categories select: [:cat | self includesSystemCategory: cat]) 
				asOrderedCollection.
	ans  remove: self name ifAbsent: [].
	^ ans asArray! !

!Package methodsFor: 'system categories' stamp: 'mist 8/22/2005 17:29'!
systemCategories
	^ SystemOrganization categories select: [:cat | self includesSystemCategory: cat]! !

!Package methodsFor: 'removing' stamp: 'ab 2/6/2005 16:37'!
unregister
	self class remove: self name! !

!Package methodsFor: 'monticello' stamp: 'ab 5/23/2005 16:20'!
updateFromSnapshot: aSnapshot
	"All the definitions contained in the snapshot have to be installed"

	(aSnapshot definitions select: [:def| def class = MCClassDefinition])
		do: [:def| |cls|
			cls := Smalltalk at: def className.
			(self includesClass: cls) ifFalse: [self addClass: cls]].

	(aSnapshot definitions select: [:def| def class = MCMethodDefinition])
		do: [:def| |cls|
				self installMethodDefinition: def].! !

!Package methodsFor: 'michal' stamp: 'mist 8/23/2005 23:53'!
updateInstVars

cache ifNil: [cache _ PackageExtensionsCache new].! !

!Package methodsFor: 'mcBrowser compatibility ' stamp: 'mist 8/21/2005 11:02'!
workingCopy
"needed by mcBrowser"

^ self mcWorkingCopy! !

!Package methodsFor: 'testing' stamp: 'ab 2/5/2005 17:10'!
= other
	^ other species = self species and: [other packageName = self packageName]! !

Object subclass: #PackageExtensionsCache
	instanceVariableNames: 'methodExtensions classExtensions'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Package-Base'!

!PackageExtensionsCache methodsFor: 'as yet unclassified' stamp: 'mist 8/23/2005 23:59'!
classExtensions

	^ classExtensions! !

!PackageExtensionsCache methodsFor: 'as yet unclassified' stamp: 'mist 8/23/2005 23:48'!
invalidate
	methodExtensions := nil.
	classExtensions := nil! !

!PackageExtensionsCache methodsFor: 'as yet unclassified' stamp: 'mist 8/23/2005 23:47'!
methodExtensions
	"Class Names as keys, and list of methods as values"

	^ methodExtensions ifNil: [methodExtensions := Dictionary new].! !

!PackageExtensionsCache methodsFor: 'as yet unclassified' stamp: 'mist 8/24/2005 00:01'!
recordClassExtensionsFor: package

	| d ans |
	d := IdentitySet new.
	ans := IdentitySet new.
	package classes do: [:c| d add: c theNonMetaClass name].

	package methodExtensions do: [:mr| (d includes: mr actualClass theNonMetaClass name) 
			ifFalse: [ans add: mr actualClass theNonMetaClass]].

	^ classExtensions := ans asArray

"	^ (package methodExtensions collect: [:mr | |cls| cls := mr actualClass theNonMetaClass]) asSet asArray
"! !

!MCMczReader methodsFor: '*Package-as yet unclassified' stamp: 'alexandrebergel 11/2/2005 13:15'!
loadPackage
	| dict |
	dict _ self parseMember: 'package'.
	package _ MCPackage named: (dict at: #name).

	(dict includesKey: #comment)
		ifTrue: [package packageInfo comment: (dict at: #comment)]! !

!MCWorkingCopy methodsFor: '*Package-accessing' stamp: 'ab 7/8/2005 08:17'!
anyPackageRequired
	"Return true or false"

	^ requiredPackages isNil not and: [requiredPackages isEmpty not]
! !

!MCWorkingCopy methodsFor: '*Package-accessing' stamp: 'ab 7/8/2005 08:13'!
isPackageRequired: aPackage
	"Return true or false"

	^ self anyPackageRequired and: [self requiredPackages includes: aPackage]! !

TestCase subclass: #PackageTest
	instanceVariableNames: 'classesCreated packagesCreated defaultRepository'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Package-Test'!

!PackageTest methodsFor: 'utility' stamp: 'ab 3/24/2005 18:22'!
createNewClassNamed: aName
	| cls |

	cls := Object subclass: aName
		instanceVariableNames: ''
		classVariableNames: ''
		poolDictionaries: ''
		category: 'TestPackage'.
	classesCreated add: cls.
	^ cls! !

!PackageTest methodsFor: 'utility' stamp: 'ab 2/8/2005 14:14'!
createNewClassNamed: aName inCategory: cat
	| cls |

	cls := Object subclass: aName
		instanceVariableNames: ''
		classVariableNames: ''
		poolDictionaries: ''
		category: cat.
	classesCreated add: cls.
	^ cls! !

!PackageTest methodsFor: 'utility' stamp: 'ab 2/6/2005 16:09'!
createNewClassNamed: aName inPackage: p
	| cls |

	cls := self createNewClassNamed: aName.
	p addClass: cls.
	^ cls! !

!PackageTest methodsFor: 'utility' stamp: 'ab 2/6/2005 16:04'!
createNewPackage: aName
	| p |
	self removePackageIfExist: aName.
	p := Package named: aName.
	packagesCreated add: p.

	^ p! !

!PackageTest methodsFor: 'utility' stamp: 'ab 2/16/2005 11:44'!
defaultRepository
	^ defaultRepository ifNil: [defaultRepository := MCCacheRepository default]! !

!PackageTest methodsFor: 'utility' stamp: 'ab 8/28/2005 13:01'!
removeClassNamedIfExists: classname
	Smalltalk at: classname ifPresent: [:c| c removeFromSystem]! !

!PackageTest methodsFor: 'utility' stamp: 'ab 8/27/2005 19:10'!
removePackageIfExist: aName
	(Package exist: aName) ifTrue: [(Package named: aName) removeFromSystem].
! !

!PackageTest methodsFor: 'running' stamp: 'ab 2/6/2005 15:48'!
setUp
	classesCreated := OrderedCollection new.
	packagesCreated := OrderedCollection new.! !

!PackageTest methodsFor: 'running' stamp: 'ab 8/28/2005 14:58'!
tearDown
	packagesCreated do: [:p| p removeFromSystem].
	(classesCreated reject: [:c| c isObsolete])
		do: [:cls| cls removeFromSystem].

	(Package exist: 'TestPackage') ifTrue: [(Package named: 'TestPackage') removeFromSystem].


	(SystemOrganization categories includes: 'MockPackage') ifTrue: [self halt].
	(SystemOrganization categories includes: 'MockPackage56b') ifTrue: [self halt].
	(SystemOrganization categories includes: 'TestPackage') ifTrue: [self halt].
	(SystemOrganization categories includes: 'Blahblahblah') ifTrue: [self halt].! !

!PackageTest methodsFor: 'class creation' stamp: 'ab 2/8/2005 19:29'!
testAutomaticSync
	"self debug: #testAutomaticSync"

	"Method created on a class should be included in the package that defines this class"

	| p cls |
	p := self createNewPackage: 'MockPackage'.
	cls := self createNewClassNamed: #Foo.
	p addClass: cls.

	self assert: (p methods size = 0).
	cls compile: 'foo ^ 10'.
	self assert: (p includesMethod: #foo ofClass: cls).

! !

!PackageTest methodsFor: 'creation/destruction' stamp: 'ab 4/10/2005 19:28'!
testCategoryRemoving
	"self debug: #testCategoryRemoving"

	| n p cls1 cls2 |
	n := 'MockPackageBLAH'.
	self removePackageIfExist: n.
	p := Package named: n.

	cls1 := Object subclass: #FooBarFoo
		instanceVariableNames: '' 
		classVariableNames: '' poolDictionaries: '' category: n.
	cls2 := Object subclass: #FooBarFoo
		instanceVariableNames: '' 
		classVariableNames: '' poolDictionaries: '' category: (n, '-Base').

	p addClass: cls1; addClass: cls2.

	p removeFromSystem.
	self assert: (SystemOrganization categoriesMatching: n, '*') isEmpty! !

!PackageTest methodsFor: 'class creation' stamp: 'ab 8/28/2005 09:45'!
testClassAddition
	"self debug: #testClassAddition"

	| p cls |
	p := self createNewPackage: 'P1'.
	cls := Object subclass: #Foo
		instanceVariableNames: ''
		classVariableNames: ''
		poolDictionaries: ''
		category: 'P1'.

	self assert: p classes size = 1.
	p addClass: cls.
	self assert: p classes size = 1.

	cls removeFromSystem.

! !

!PackageTest methodsFor: 'class creation' stamp: 'ab 8/28/2005 15:26'!
testClassCreation
	"self debug: #testClassCreation"

	| n p cls |
	n := 'MockPackage'.
	self removePackageIfExist: n.
	p := Package named: n.

	cls := self createNewClassNamed: #Foo.
	self assert: (p includesClassNamed: #Foo) not.
	self assert: (p includesClass: cls) not.

	p addClass: cls.

	self assert: (p includesClassNamed: #Foo).
	self assert: (p includesClass: cls).
	self assert: (p includesClass: cls class).

	p removeFromSystem.! !

!PackageTest methodsFor: 'creation/destruction' stamp: 'ab 2/9/2005 23:17'!
testClassDestruction
	"self debug: #testClassDestruction"

	| p cls |
	p := self createNewPackage: 'FooBar'.
	self assert: (p classes size = 0).
	self assert: (p methods size = 0).

	cls := Object subclass: #FooBarClass
		instanceVariableNames: ''
		classVariableNames: ''
		poolDictionaries: ''
		category: 'FooBar'.
	cls compile: 'foo ^ 10'.

	self assert: (p classes size = 1).
	self assert: (p methods size = 1).

	cls removeFromSystem.
	self assert: (p classes size = 0).
	self assert: (p methods size = 0).! !

!PackageTest methodsFor: 'class creation' stamp: 'ab 5/12/2005 16:07'!
testClassExtension
	"self debug: #testClassExtension"

	| p1 p2 cls2 p3 cls3 |
	p1 := self createNewPackage: 'MockPackage1'.

	p2 := self createNewPackage: 'MockPackage2'.
	cls2 := self createNewClassNamed: #Foo inPackage: p2.

	p3 := self createNewPackage: 'MockPackage3'.
	cls3 := self createNewClassNamed: #Foo2 inPackage: p3.

	cls2 compile: 'foo ^ 10'.
	cls2 compile: 'bar ^ self foo'.
	p1 addMethod: (cls2>>#foo) methodReference.
	p1 addMethod: (cls2>>#bar) methodReference.

	cls3 compile: 'foo ^ 10'.
	cls3 compile: 'bar ^ self foo'.
	p1 addMethod: (cls3>>#foo) methodReference.
	p1 addMethod: (cls3>>#bar) methodReference.

	self assert: (p1 methodExtensions size = 4).
	self assert: (p1 listOfMethodExtensions = {{p2. cls2. #foo} . {p2. cls2. #bar} .
											{p3. cls3. #foo} . {p3. cls3. #bar}}).
	self assert: ((p1 listOfExtendedClasses = {cls2 . cls3}) or:
					[p1 listOfExtendedClasses = {cls3 . cls2}]).
	self assert: (p1 listOfMethodExtensionsFor: cls2) asArray = {#foo . #bar}
! !

!PackageTest methodsFor: 'class removing' stamp: 'ab 8/28/2005 09:44'!
testClassRemoving
	"self debug: #testClassRemoving"

	| p cls |
	p := self createNewPackage: 'P1'.
	cls := Object subclass: #Foo
		instanceVariableNames: ''
		classVariableNames: ''
		poolDictionaries: ''
		category: 'P1'.

	cls compile: 'foo ^ 10'.

	self assert: (Smalltalk at: #Foo) new foo = 10.
	self assert: p methods size = 1.
	self assert: (p includesMethod: #foo ofClass: cls).
	self assert: p classes size = 1.
	self assert: (p includesClass: cls).

	cls removeFromSystem.
	self assert: p methods size = 0.
	self assert: p classes size = 0.

! !

!PackageTest methodsFor: 'class creation' stamp: 'ab 8/28/2005 10:50'!
testClassRenaming
	"self debug: #testClassRenaming"

	| class p p2 |
	p := self createNewPackage: 'MockClassRenaming'.
	class := self createNewClassNamed: #FooBar inPackage: p.
	class compile: 'foo ^10'.

	self assert: (p classes size = 1).
	self assert: (p includesClassNamed: #FooBar).
	self assert: (p includesClass: class).

	self assert: (p methods size = 1).
	self assert: (p methods first classSymbol == #FooBar).

	class rename: #FooBarBar.
	self assert: (p includesClassNamed: #FooBar) not.
	self assert: (p includesClassNamed: #FooBarBar).
	self assert: (p includesClass: class).

	self assert: (p methods size = 1).
	self assert: (p includesMethod: #foo ofClass: class).
	self assert: (p methods first classSymbol == #FooBarBar).

	"---------"
	"A new package now extend the class"
	p2 := self createNewPackage: 'MockClassRenaming2'.
	class compile: 'bar ^ self foo'.
	p moveMethods: (class >> #bar) methodReference  to: p2.

	self assert: (p2 includesMethod: #bar ofClass: class).
	class rename: #FooBar.
	self assert: (p2 includesMethod: #bar ofClass: class).
	self assert: (p2 methods first classSymbol == #FooBar).	
	self assert: (p2 methods first methodSymbol == #bar).	

	self assert: (p methods size = 1).
	self assert: (p includesMethod: #foo ofClass: class).
	self assert: (p methods first classSymbol == #FooBar).
! !

!PackageTest methodsFor: 'comment' stamp: 'alexandrebergel 11/2/2005 11:19'!
testCommentWithFlush
	"self debug: #testCommentWithFlush"

	"Before loading the package, we do a flush. As a consequence, the file is read from the disk"

	| p wc name version newPVersion |
	self removePackageIfExist: #P.
	p := Package named: #P.

	self assert: (p hasComment not).
	p comment: 'my comment'.

	wc := p mcWorkingCopy.
	name := wc uniqueVersionName.
	version := wc newVersionWithName: name message: 'Blah blah'.

	self defaultRepository storeVersion: version.

	self defaultRepository flushCache. 

	"-----------"
	p removeFromSystem.
	p := nil.
	"-----------"
	newPVersion := self defaultRepository versionFromFileNamed: (name, '.mcz').
	newPVersion workingCopy modified: false.
	newPVersion load.

	p := Package named: #P.
	self assert: p name = #P.
	self assert: p hasComment.
	self assert: p comment = 'my comment'.

	p removeFromSystem! !

!PackageTest methodsFor: 'comment' stamp: 'alexandrebergel 11/2/2005 11:20'!
testCommentWithoutFlush
	"self debug: #testCommentWithoutFlush"

	"Before loading the package, we do not flush the repository. 
	As a consequence, the file is read from the cache contained in the image"

	| p wc name version newPVersion |
	self removePackageIfExist: #P.
	p := Package named: #P.

	self assert: (p hasComment not).
	p comment: 'my comment'.

	wc := p mcWorkingCopy.
	name := wc uniqueVersionName.
	version := wc newVersionWithName: name message: 'Blah blah'.

	self defaultRepository storeVersion: version.


	"-----------"
	p removeFromSystem.
	p := nil.
	"-----------"
	newPVersion := self defaultRepository versionFromFileNamed: (name, '.mcz').
	newPVersion workingCopy modified: false.
	newPVersion load.

	p := Package named: #P.
	self assert: p name = #P.
	self assert: p hasComment.
	self assert: p comment = 'my comment'.

	p removeFromSystem! !

!PackageTest methodsFor: 'creation/destruction' stamp: 'ab 2/6/2005 15:50'!
testCreation
	"self debug: #testCreation"

	| n |
	n := 'MockPackage'.
	self removePackageIfExist: n.
	self assert: (Package exist: n) not.
	Package named: 'MockPackage'.
	self assert: (Package exist: n).

	Package remove: n.
	self assert: (Package exist: n) not.! !

!PackageTest methodsFor: 'creation/destruction' stamp: 'ab 2/8/2005 13:35'!
testCreation2
	"self debug: #testCreation2"

	| n p1 p2 |
	n := 'MockPackage'.
	p1 := Package named: n.
	p2 := Package named: n.
	self assert: (p1 == p2).

	Package remove: n.
	self assert: (Package exist: n) not.! !

!PackageTest methodsFor: 'class creation' stamp: 'ab 8/28/2005 15:07'!
testCreationFromCategoryCreation
	"self debug: #testCreationFromCategoryCreation"

	self removePackageIfExist: 'MockPackage56b'.
	self assert: (Package packages includesKey: #MockPackage56b) not.

	self createNewClassNamed: #FooBar56b inCategory: 'MockPackage56b'.
	self assert: (Package exist: #MockPackage56b).

	self removePackageIfExist: 'MockPackage56b'.! !

!PackageTest methodsFor: 'creation/destruction' stamp: 'ab 8/28/2005 15:09'!
testDestruction
	"self debug: #testDestruction"

	| n1 n2 p1 p2 cls |
	n1 := 'MockPackage1'.
	n2 := 'MockPackage2'.
	self removePackageIfExist: n1.
	self removePackageIfExist: n2.
	p1 := Package named: n1.
	p2 := Package named: n2.

	cls := Object subclass: #FooBarFoo
		instanceVariableNames: '' 
		classVariableNames: '' poolDictionaries: '' category: 'Blahblahblah'.
	cls compile: 'foo ^10'.
	p1 addClass: cls.
	p2 addMethod: (cls>>#foo).

	self assert: (p1 includesClass: cls).
	self assert: (p2 includesClass: cls) not.
	self assert: (p1 includesMethod: #foo ofClass: cls) not.
	self assert: (p2 includesMethod: #foo ofClass: cls).

	self assert: (cls canUnderstand: #foo).
	p2 removeFromSystem.
	self assert: (cls canUnderstand: #foo) not.

	p1 removeFromSystem.
	(Package named: 'Blahblahblah') removeFromSystem.
	self assert: (Smalltalk includesKey: #FooBarFoo) not! !

!PackageTest methodsFor: 'class creation' stamp: 'ab 2/22/2005 11:25'!
testMethodCreation
	"self debug: #testMethodCreation"

	| p cls |
	p := self createNewPackage: 'MockPackage'.
	cls := self createNewClassNamed: #Foo inPackage: p.
	cls compile: 'foo ^ 10'.
	p addMethod: (cls>>#foo) methodReference.

	self assert: (p includesMethod: #foo ofClass: cls).

	self assert: (cls>>#foo) methodReference packageDefining == p.! !

!PackageTest methodsFor: 'methods' stamp: 'ab 8/28/2005 14:24'!
testMethodRecategorization
	"self debug: #testMethodRecategorization"

	| cls p |
	self removePackageIfExist: 'MockPackage'.
	self assert: (Package exist: 'MockPackage') not.

	cls := Object subclass: #FooBar
			instanceVariableNames: ''
			classVariableNames: ''
			poolDictionaries: ''
			category: 'MockPackage'.

	p := Package named: 'MockPackage'.

	cls compile: 'zork ^ 10'.
	self assert: (p includesMethod: #zork ofClass: cls).

	cls compile: 'zork ^ 10' classified: '*MockPackage'.
	self assert: (p includesMethod: #zork ofClass: cls).

	cls compile: 'zork ^ 10' classified: 'blahblah'.
	self assert: (p includesMethod: #zork ofClass: cls).


	self removePackageIfExist: 'MockPackage'.
	self removePackageIfExist: 'AnotherPackage'.
	! !

!PackageTest methodsFor: 'operators' stamp: 'ab 7/11/2005 12:40'!
testMoveMethod
	"self debug: #testMoveMethod"

	| p1 p2 p3 cls |
	p1 := self createNewPackage: 'P1'.
	p2 := self createNewPackage: 'P2'.
	p3 := self createNewPackage: 'P3'.

	cls := self createNewClassNamed: #Foo inPackage: p1.
	cls compile: 'foo ^10'.

	self assert: (p1 includesClass: cls).
	self assert: (p1 includesMethod: #foo ofClass: cls).

	self assert: (p2 includesClass: (Smalltalk at: #Foo)) not.
	self assert: (p2 includesMethod: #foo ofClass: cls) not.

	self assert: (p1 listOfExtendedClasses = {}).
	self assert: (p2 listOfExtendedClasses = {}).
	self assert: (p3 listOfExtendedClasses = {}).

	"--------------"
	p1 moveMethods: (cls>>#foo) methodReference to: p2.

	self assert: (p1 includesClass: cls).
	self assert: (p1 includesMethod: #foo ofClass: cls) not.

	self assert: (p2 includesClass: cls) not.
	self assert: (p2 includesMethod: #foo ofClass: cls).

	self assert: (p1 listOfExtendedClasses = {}).
	self assert: (p2 listOfExtendedClasses = {cls}).
	self assert: (p3 listOfExtendedClasses = {}).

	"-------------"
	"However, method can be moved from p1"
	p1 moveMethods: (cls>>#foo) methodReference to: p3.
	self assert: (p1 includesClass: cls).
	self assert: (p2 includesClass: cls) not.
	self assert: (p3 includesClass: cls) not.
	self assert: (p1 includesMethod: #foo ofClass: cls) not.
	self assert: (p2 includesMethod: #foo ofClass: cls) not.
	self assert: (p3 includesMethod: #foo ofClass: cls).

	self assert: (p1 listOfExtendedClasses = {}).
	self assert: (p2 listOfExtendedClasses = {}).
	self assert: (p3 listOfExtendedClasses = {cls}).! !

!PackageTest methodsFor: 'methods' stamp: 'ab 8/28/2005 13:44'!
testPackageBelongingMethod
	"self debug: #testPackageBelongingMethod"

	| cls p p2 |
	self removePackageIfExist: 'MockPackage'.
	self removeClassNamedIfExists: #FooBar.

	cls := Object subclass: #FooBar
			instanceVariableNames: ''
			classVariableNames: ''
			poolDictionaries: ''
			category: 'MockPackage'.

	cls compile: 'foo ^ 10'.

	"A new class was created, therefore the package has to exist"
	p := Package named: 'MockPackage'.

	"The system can infer to which package this method has to be included" 
	self assert: (Package packageBelongingMethodNamed: #foo inClass: cls) = p.

	"Everything is alright"
	self assert: (p includesMethod: #foo ofClass: cls).

	"-----------"
	"We compile a **new** method that belongs to another package"
	cls compile: 'bar ^ self foo' classified: '*AnotherMockPackage'.

	"The package should now exists"
	self assert: (Package exist: 'AnotherMockPackage').
	p2 := Package named: 'AnotherMockPackage'.

	"Inference of the package"
	self assert: (Package packageBelongingMethodNamed: #bar inClass: cls) = p2.
	self assert: (p2 includesMethod: #bar ofClass: cls).

	"-----------"
	"We **recategorize** the foo method"
	cls compile: 'foo ^ 10' classified: '*AnotherMockPackage'.

	"Inference of the package"
	self assert: (Package packageBelongingMethodNamed: #foo inClass: cls) = p2.
	self assert: (p2 includesMethod: #foo ofClass: cls).

	self assert: (p includesMethod: #foo ofClass: cls) not.
	self assert: (p methods isEmpty).

	"-----------"

	self removeClassNamedIfExists: #FooBar.
	p removeFromSystem.
	p2 removeFromSystem
! !

!PackageTest methodsFor: 'creation/destruction' stamp: 'ab 5/12/2005 15:52'!
testRemovingAnExtendedPackage
	"self debug: #testRemovingAnExtendedPackage"

	"This test set up two packages, p1 and p2.
	p1 define a class:
		A
	p2 extends p1 such as:
		A>>foo

	removing p1 should also remove A>>foo in p2"

	| p1 p2 cls |
	p1 := Package named: 'P1'.
	p2 := self createNewPackage: 'P2'.

	p1 addClass: (cls := self createNewClassNamed: #A).
	cls compile: 'foo ^ 10'.
	p2 addMethod: ((cls >> #foo) methodReference).

	self assert: (p2 doesDefineMethodForClass: cls).
	self assert: (p1 packagesExtendingClass: cls) = {p2}.

	self assert: p2 methods size = 1.

	"------------"
	p1 removeFromSystem.
	self assert: p2 methods size = 0.! !

!PackageTest methodsFor: 'operators' stamp: 'ab 2/10/2005 15:28'!
testRename

	| p cls1 cls2 |
	p := self createNewPackage: 'MockPackage'.
	self assert: (p name = 'MockPackage').

	cls1 := Object subclass: #FooBarFoo
		instanceVariableNames: ''
		classVariableNames: ''
		poolDictionaries: ''
		category: 'MockPackage'.
	cls2 := Object subclass: #FooBarFoo2
		instanceVariableNames: ''
		classVariableNames: ''
		poolDictionaries: ''
		category: 'MockPackage-Base'.

	p addClass: cls1.
	p addClass: cls2.

	p rename: 'NewMockPackage'.

	self assert: (p name = 'NewMockPackage').
	self assert: (cls1 category = 'NewMockPackage').
	self assert: (cls2 category = 'NewMockPackage-Base').! !

!PackageTest methodsFor: 'requirement' stamp: 'ab 7/8/2005 08:14'!
testRequirement
	"self debug: #testRequirement"

	| p1 p2 |
	p1 := self createNewPackage: #P1.
	p2 := self createNewPackage: #P2.

	self assert: p1 anyPackageRequired not.
	self assert: p2 anyPackageRequired not.

	p1 requirePackage: p2.

	self assert: p1 anyPackageRequired.
	self assert: p2 anyPackageRequired not.

	self assert: (p1 isPackageRequired: p2).
	self assert: (p2 isPackageRequired: p1) not.! !

!PackageTest methodsFor: 'repository' stamp: 'ab 5/12/2005 11:51'!
testSavingAndLoading
	"self debug: #testSavingAndLoading"

	"Two packages, p1 and p2, are setup.
	p1 defines:
		FooBarClass
			foo
			bar
		FooBarClass2>>foo  classified: 'blahblah' 
	p2 defines:
		FooBarClass2
	"

	| p1 cls1 p2 cls2 wc version name newP1Version newP1 |
	self removePackageIfExist: 'FooBar'.
	p1 := Package named: 'FooBar'.
	cls1 := self createNewClassNamed: #FooBarClass inPackage: p1.
	cls1 compile: 'foo ^10' classified: 'blah'.
	cls1 compile: 'bar ^self foo'.
	p1 addMethod: (cls1 >> #foo) methodReference.
	p1 addMethod: (cls1 >> #bar) methodReference.

	p2 := self createNewPackage: 'FooBar2'.
	cls2 := self createNewClassNamed: #FooBarClass2 inPackage: p2.
	cls2 compile: 'foo ^ 20' classified: 'blahblah'.

	p1 addMethod: (cls2 >> #foo) methodReference.
	self assert: (p1 includesMethod: #foo ofClass: cls2).
	self assert: (cls2 >> #foo) methodReference category = 'blahblah'.

	wc := p1 mcWorkingCopy.
	name := wc uniqueVersionName.
	version := wc newVersionWithName: name message: 'Blah blah'.
	self defaultRepository storeVersion: version.

	"-----------"
	p1 removeFromSystem.
	self assert: (Package exist: 'FooBar') not.
	self assert: (Smalltalk includesKey: #FooBarClass) not.
	self assert: (Smalltalk includesKey: #FooBarClass2).
	"-----------"
	newP1Version := self defaultRepository versionFromFileNamed: (name, '.mcz').
	self assert: ((newP1Version snapshot definitions) 
				select: [:def| (def class == MCMethodDefinition) 
							and: [def className == #FooBarClass2]]) first category 
				= '*FooBar-blahblah'.

	newP1Version workingCopy modified: false.
	newP1Version load.
	newP1 := Package named: 'FooBar'.

	self assert: (Smalltalk includesKey: #FooBarClass).
	self assert: newP1 name = 'FooBar'.
	self assert: newP1 classes size = 1.
	self assert: newP1 classes asOrderedCollection first == (Smalltalk at: #FooBarClass).
	self assert: newP1 methods size = 3.

	self assert: (newP1 includesMethod: #foo ofClass: (Smalltalk at: #FooBarClass)).
	self assert: ((Smalltalk at: #FooBarClass2) >> #foo) methodReference category = 'blahblah'.
	self assert: ((Smalltalk at: #FooBarClass) >> #foo) methodReference category = 'blah'.

	newP1 removeFromSystem.
	! !

!PackageTest methodsFor: 'sub category' stamp: 'ab 4/10/2005 19:17'!
testSubcategory1
	"self debug: #testSubcategory1"

	| p cls |
	p := self createNewPackage: 'P1'.
	self assert: (p subcategories isEmpty).

	cls := self createNewClassNamed: #Foo inPackage: p.
	self assert: (p subcategories isEmpty).

	self assert: (p classesOfSubcategory: '') asArray = {cls}.
	self assert: (p classesOfSubcategory: 'Bouba') asArray = #().
	self assert: (p classesOfSubcategory: 'Boubi') asArray = #().

	self assert: (p subCategoryOf: #Foo) = ''.
	p moveClassNamed: #Foo inSubcategoryNamed: 'Bouba'.
	self assert: (p classesOfSubcategory: '') asArray = #().
	self assert: (p classesOfSubcategory: 'Bouba') asArray = {cls}.
	self assert: (p classesOfSubcategory: 'Boubi') asArray = #().

	self assert: (p subCategoryOf: #Foo) = 'Bouba'.
	p moveClassNamed: #Foo inSubcategoryNamed: 'Boubi'.
	self assert: (p classesOfSubcategory: '') asArray = #().
	self assert: (p classesOfSubcategory: 'Bouba') asArray = #().
	self assert: (p classesOfSubcategory: 'Boubi') asArray = {cls}.

	self assert: (p subCategoryOf: #Foo) = 'Boubi'.
	p moveClassNamed: #Foo inSubcategoryNamed: ''.
	self assert: (p classesOfSubcategory: '') asArray = {cls}.
	self assert: (p classesOfSubcategory: 'Bouba') asArray = #().
	self assert: (p classesOfSubcategory: 'Boubi') asArray = #().

	self assert: (p subCategoryOf: #Foo) = ''.
! !

!MCMethodDefinition methodsFor: '*Package-packages' stamp: 'ab 5/5/2005 23:44'!
asMethodReference
	^ MethodReference class: self actualClass selector: self selector	! !

!MCMethodDefinition methodsFor: '*Package-packages' stamp: 'ab 5/5/2005 23:40'!
category: newCategory
	"Insure compatibility while using PackageBrowser"
	"This method is called to set the category of a class extension according to 
	the convention *extendingPackage-category"

	category := newCategory! !

!MCMethodDefinition methodsFor: '*Package-packages' stamp: 'ab 5/5/2005 23:46'!
isClassExtensionOf: package
	^ (package includesClass: self actualClass) not 
		and: [package includesMethod: self selector ofClass: self actualClass]! !

!MCStReader methodsFor: '*Package' stamp: 'alexandrebergel 11/2/2005 11:57'!
packageCommentsFromRecords: changeRecords
	| comments |
	comments _ changeRecords
					select: [:ea | 'Smalltalk at: #Package*' match: ea string]
					thenCollect: [:ea | (self packageCommentFromDoIt: ea string)].
	^ comments isEmpty ifFalse: [MCPackageCommentDefinition comments: comments asArray]! !

!SystemChangeNotifier methodsFor: '*Package-system triggers' stamp: 'ab 2/10/2005 16:32'!
classCategoryAdded: aClassCategoryName

	self trigger: (AddedEvent
					classCategory: aClassCategoryName)! !

!SystemChangeNotifier methodsFor: '*Package-system triggers' stamp: 'ab 2/10/2005 16:35'!
classCategoryRemoved: aClassCategoryName

	self trigger: (RemovedEvent
					classCategory: aClassCategoryName)! !

!SystemChangeNotifier methodsFor: '*Package-system triggers' stamp: 'ab 2/10/2005 16:44'!
classCategoryRenamedFrom: anOldClassCategoryName to: aNewClassCategoryName

	self trigger: (RenamedEvent
					classCategoryRenamedFrom: anOldClassCategoryName 
					to: aNewClassCategoryName)! !

!SystemChangeNotifier methodsFor: '*Package-system triggers' stamp: 'ab 2/14/2005 11:13'!
packageAdded: aPackage

	self trigger: (AddedEvent
					package: aPackage)! !

!SystemChangeNotifier methodsFor: '*Package-system triggers' stamp: 'ab 2/14/2005 11:13'!
packageRemoved: aPackage

	self trigger: (RemovedEvent
					package: aPackage)! !

!SystemChangeNotifier methodsFor: '*Package-system triggers' stamp: 'ab 2/14/2005 11:25'!
packageRenamed: aPackage

	"We should also pass the old and new name..."

	self trigger: (RenamedEvent
					package: aPackage)! !

!PackageInfo methodsFor: '*Package-migration' stamp: 'mist 8/24/2005 13:46'!
asPackage

	^ Package fromPackageInfo: self! !

!MCVersionLoader methodsFor: '*Package-loading' stamp: 'ab 2/16/2005 22:36'!
load
	| loader packageName package |
	self checkForModifications.
	loader _ MCPackageLoader new.
	versions do: [:ea | loader updatePackage: ea package withSnapshot: ea snapshot].
	loader loadWithNameLike: versions first info name.
	versions do: [:ea | 
					ea workingCopy loaded: ea.
					packageName := ea package name.
					package := Package named: packageName.
					package updateFromSnapshot: ea snapshot].! !

!AbstractEvent class methodsFor: '*Package-instance creation' stamp: 'NS 1/19/2004 18:42'!
class: aClass category: cat 
	| instance |
	instance := self class: aClass.
	instance itemCategory: cat.
	^instance! !

!AbstractEvent class methodsFor: '*Package-item kinds' stamp: 'ab 2/14/2005 11:10'!
packageKind

	^#package! !

!AbstractEvent class methodsFor: '*Package-instance creation' stamp: 'ab 2/14/2005 11:09'!
package: aPackage
	^ self item: aPackage kind: AbstractEvent packageKind.! !

MCDefinition subclass: #MCPackageCommentDefinition
	instanceVariableNames: 'packageName comment'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Package-Base'!

!MCPackageCommentDefinition class methodsFor: 'as yet unclassified' stamp: 'alexandrebergel 11/2/2005 13:08'!
fromPackage: aPackage
	^ self new
		package: aPackage;
		yourself! !

!MCPackageCommentDefinition methodsFor: 'as yet unclassified' stamp: 'alexandrebergel 11/2/2005 12:03'!
accept: aVisitor
	^ aVisitor visitPackageCommentDefinition: self! !

!MCPackageCommentDefinition methodsFor: 'as yet unclassified' stamp: 'alexandrebergel 11/2/2005 13:11'!
comment
	^ comment! !

!MCPackageCommentDefinition methodsFor: 'as yet unclassified' stamp: 'alexandrebergel 11/2/2005 12:04'!
comment: aString
	comment := aString! !

!MCPackageCommentDefinition methodsFor: 'as yet unclassified' stamp: 'alexandrebergel 11/2/2005 13:09'!
description
	^ Array with: packageName with: comment! !

!MCPackageCommentDefinition methodsFor: 'as yet unclassified' stamp: 'alexandrebergel 11/2/2005 13:13'!
load
	(Package named: self packageName) comment: self comment! !

!MCPackageCommentDefinition methodsFor: 'as yet unclassified' stamp: 'alexandrebergel 11/2/2005 13:11'!
packageName
	^ packageName! !

!MCPackageCommentDefinition methodsFor: 'as yet unclassified' stamp: 'alexandrebergel 11/2/2005 12:03'!
packageName: aPackageName
	packageName := aPackageName! !

!MCPackageCommentDefinition methodsFor: 'as yet unclassified' stamp: 'alexandrebergel 11/2/2005 12:05'!
package: aPackage
	self packageName: aPackage name.
	self comment: aPackage comment! !

!MCPackageCommentDefinition methodsFor: 'as yet unclassified' stamp: 'alexandrebergel 11/2/2005 13:10'!
summary
	^ packageName ! !

!ClassDescription methodsFor: '*Package-packages' stamp: 'ab 2/22/2005 18:07'!
package

	^ Package packageDefining: self theNonMetaClass! !

!RemovedEvent class methodsFor: '*Package-accessing' stamp: 'ab 2/14/2005 19:38'!
supportedKinds
	"All the kinds of items that this event can take."
	
	^ Array with: self classKind with: self methodKind with: self categoryKind with: self protocolKind with: self packageKind! !

!MCStWriter methodsFor: '*Package' stamp: 'alexandrebergel 11/2/2005 12:00'!
visitPackageCommentDefinition: definition
	self writePackageComment: definition.! !

!MCStWriter methodsFor: '*Package' stamp: 'alexandrebergel 11/2/2005 11:50'!
writePackageComment: definition
	stream
		cr;
		nextPut: $!!;
		nextPutAll: 'Smalltalk at: #Package ifPresent:';
		nextPutAll: '  [:p| (p named: ''', definition packageName, ''')';
		nextPutAll: '           comment: ''', definition comment, ''']';
		nextPutAll: '!!';
		cr! !

!RenamedEvent class methodsFor: '*Package-instance creation' stamp: 'ab 2/10/2005 16:49'!
classCategoryRenamedFrom: anOldClassCategoryName to: aNewClassCategoryName

	^(self classCategory: anOldClassCategoryName) oldName: anOldClassCategoryName; newName: aNewClassCategoryName! !

!RenamedEvent class methodsFor: '*Package-accessing' stamp: 'ab 2/14/2005 19:38'!
supportedKinds
	"All the kinds of items that this event can take."
	
	^ Array with: self classKind with: self categoryKind with: self protocolKind with: self packageKind! !

!Behavior methodsFor: '*Package-package' stamp: 'ab 2/8/2005 19:37'!
package
	^ Package packageDefining: self! !
Package initialize!
